// -*- C++ -*- std header                                                      |
//-----------------------------------------------------------------------------+
//                                                                             |
// kern.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2001                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie,                               |
//           Knigstuhl 17, D-69117 Heidelberg, Germany                        |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// defines                                                                     |
//                                                                             |
//                                                                             |
//-----------------------------------------------------------------------------+

#ifndef included_kern_h
#define included_kern_h 1

#ifndef included_iostream
#include <iostream>
#define included_iostream
#endif

namespace nbdy {
  //////////////////////////////////////////////////////////////////////////////
  // enum nbdy::tree_soft_type                                                  
  //////////////////////////////////////////////////////////////////////////////
  enum soft_type {                         // type of softening                 
    global      = 0,                       // global eps                        
    individual  = 1                        // use individual eps                
  };                                       //                                   
  //////////////////////////////////////////////////////////////////////////////
  // enum nbdy::kern_type                                                       
  //////////////////////////////////////////////////////////////////////////////
  enum kern_type {                         // type of kernel                    
    newton,                                // no softening                      
    f0, f1, f2, f3,                        // Ferrers sphere of index n=0,1,2,3 
    k0, k1, k2, k3,                        // compensating kernels    n=0,1,2,3 
    p0, p1, p2, p3                         // Plummer family of kernels         
  };                                       // (p0=standard Plummer softening)   
  //----------------------------------------------------------------------------
  inline nbdy::kern_type kern(const int KERN) { 
    switch(KERN) { 
    case  0: return f0;
    case  1: return f1;
    case  2: return f2;
    case  3: return f3;
    case 10: return k0;
    case 11: return k1;
    case 12: return k2;
    case 13: return k3;
    case 20: return p0;
    case 21: return p1;
    case 22: return p2;
    case 23: return p3;
    default: std::cerr<<" WARNING: kernel unknown, defaulting to Newtonian\n";
      return nbdy::newton;
    }
  }
}
//------------------------------------------------------------------------------
#endif // included_kern_h
