// -*- C++ -*- std header                                                      |
//-----------------------------------------------------------------------------+
//                                                                             |
// nbio.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2001                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie,                               |
//           Knigstuhl 17, D-69117 Heidelberg, Germany                        |
//                                                                             |
//-----------------------------------------------------------------------------+
#ifndef included_nbio_h
#define included_nbio_h
////////////////////////////////////////////////////////////////////////////////
namespace nbdy {

  class io {
    int bits;
  public:
    enum { no  = 0,                                // nothing written/to be read
	   m   = 1 << 0,                           // masses                    
	   x   = 1 << 1,                           // positions                 
	   v   = 1 << 2,                           // velocities                
	   e   = 1 << 3,                           // indiv eps_i               
	   p   = 1 << 4,                           // N-body potentials         
	   a   = 1 << 5,                           // accelarations             
	   r   = 1 << 6,                           // density                   
	   au  = 1 << 7,                           // auxiliary data            
	   pe  = 1 << 8,                           // external potentials       
	   xv  = x   | v,                          // phases = [x,v]            
	   mxv = m   | xv,                         // masses & phases           
	   mxve= mxv | e  };                       // masses, phases & eps_i    
    //--------------------------------------------------------------------------
    io()            : bits(io::no) {}
    io(const int i) : bits(i) {}
    io(const io& i) : bits(i.bits) {}
    //--------------------------------------------------------------------------
    io&  operator=     (const io &i)       { bits =i.bits; return *this; }
    io&  operator|=    (const io &i)       { bits|=i.bits; return *this; }
    io&  operator|=    (const int&i)       { bits|=i; return *this; }
    io&  operator&=    (const io &i)       { bits&=i.bits; return *this; }
    io&  operator&=    (const int&i)       { bits&=i; return *this; }
    operator int&      ()                  { return bits; }
    //--------------------------------------------------------------------------
    io   operator|     (const io &i) const { return io(bits | i.bits); }
    io   operator|     (const int&i) const { return io(bits | i); }
    io   operator&     (const io &i) const { return io(bits & i.bits); }
    io   operator&     (const int&i) const { return io(bits & i); }
    bool operator==    (const io &i) const { return bits == i.bits; }
    bool operator==    (const int&i) const { return bits == i; }
    bool operator!=    (const io &i) const { return bits != i.bits; }
    bool operator!=    (const int&i) const { return bits != i; }
    io   operator~     ()            const { return io(~bits); }
  };
}
////////////////////////////////////////////////////////////////////////////////
#endif // included_nbio_h
