// -*- C++ -*-                                                                 |
//-----------------------------------------------------------------------------+
//                                                                             |
// ionl.h                                                                      |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 1994-2001                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie,                               |
//           Knigstuhl 17, D-69117 Heidelberg, Germany                        |
//                                                                             |
//-----------------------------------------------------------------------------+

#ifndef included_ioln_h
#define included_ioln_h

#ifndef included_fstream
#include <fstream>
#define included_fstream
#endif

//------------------------------------------------------------------------------
namespace util {
  //----------------------------------------------------------------------------
  inline bool open(std::ofstream& S, const char* file,
		   std::ios::openmode mode = std::ios::out | std::ios::trunc)
  {
    S.open(file,mode);
    if(! S.is_open() ) {
      std::cerr<<" cannot open file \""<<file<<"\" for output";
      return false;
    }
    return true;
  }
  //----------------------------------------------------------------------------
  inline bool open_to_append(std::ofstream& S, const char* file)
  {
    S.open(file,std::ios::out | std::ios::app);
    if(! S.is_open() ) S.open(file,std::ios::out);
    if(! S.is_open() ) {
      std::cerr<<" cannot open file \""<<file<<"\" for appending";
      return false;
    }
    return true;
  }
  //----------------------------------------------------------------------------
  inline bool open(std::ifstream& S, const char* file,
		  std::ios::openmode mode=std::ios::in)
  {
    S.open(file,mode);
    if(! S.is_open() ) {
      std::cerr<<" cannot open file for \""<<file<<"\" input";
      return 0;
    }
    return 1;
  }
  //----------------------------------------------------------------------------
  inline bool open(std::fstream& S, const char* file, std::ios::openmode mode)
  {
    S.open(file,mode);
    if(! S.is_open() ) {
      std::cerr<<" cannot open file \""<<file<<"\"";
      return 0;
    }
    return 1;
  }
  //----------------------------------------------------------------------------
  inline void SwallowRestofLine(std::istream& from)
  {
    char c;
    do from.get(c); while( from.good() && c !='\n');
  }
  //----------------------------------------------------------------------------
  inline const char* stndrdth(const int i)
  {
    register int ia= (i<0)? -i:i;
    switch( ia % 100 ) {
    case 11: 
    case 12: 
    case 13: return "th";
    default:
      switch( ia % 10 ) {
      case 1:  return "st";
      case 2:  return "nd"; 
      case 3:  return "rd";
      default: return "th";
      }   
    }
  }
}
//------------------------------------------------------------------------------
#endif // included_ioln_h
