//-----------------------------------------------------------------------------+
//                                                                             |
// tayl.cc                                                                     |
//                                                                             |
// C++ code                                                                    |
//                                                                             |
// Copyright Walter Dehnen, 2000-2001                                          |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie,                               |
//           Knigstuhl 17, D-69117 Heidelberg, Germany                        |
//                                                                             |
//-----------------------------------------------------------------------------+

#include <nbdy/tayl.h>
#include <nbdy/soul.h>

using namespace nbdy;
////////////////////////////////////////////////////////////////////////////////
// class nbdy::TaylorCoeffs                                                     
////////////////////////////////////////////////////////////////////////////////

TaylorCoeffs& TaylorCoeffs::shift_by(const vect& dX)
{
  const real sixth=0.1666666666666666667;
  if(dX==zero) return *this;
  if(c0() != zero) {
    real          &P(c0());
    register evec C1(G+1);                         // *   +/-  =      cumulative
    register emat C2(G+N_C2);                      // --------------------------
    register matr dXC3   = dX*c3();                // 18  12   6      18  12   6
    register vect dXC2   = dX*C2,                  //  9   6   3      27  18   9
                  dXdXC3 = dX*dXC3,                //  9   6   3      36  24  12
                  T      = C1;                     //  0   0   3       -   -  15
    T    += C1;                                    //  0   3   0       -  27   -
    T    += dXC2;                                  //  0   3   0       -  30   -
    T    *= 3;                                     //  3   0   0      39   -   -
    T    += dXdXC3;                                //  0   3   0       -  33   -
    P    += sixth*(dX*T);                          //  4   1   0      43  34   -
    C1   += dXC2;                                  //  0   3   0       -  37   -
    C1   += half*dXdXC3;                           //  3   3   0      46  40   -
    C2   += dXC3;                                  //  0   6   0       -  46   -
  }
  return *this;
}
//------------------------------------------------------------------------------
void TaylorCoeffs::extract_grav(const vect& dX, soul* S) const
{
  register matr dXdX;                              // *   +/-  =      cumulative
  register vect                                    // --------------------------
  dF1      = dX*nbdy::c2(this),                    //  9   6   3       9   6   3
  dF2      = nbdy::c3(this)*(dXdX.outprod(dX));    // 24  15  15      33  21  18
  dF2     *= half;                                 //  3   0   0      36   -   -
  S->acc()+= nbdy::c1(this);                       //  0   3   0       -  24   -
  S->acc()+= dF1;                                  //  0   3   0       -  27   -
  S->acc()+= dF2;                                  //  0   3   0       -  30   -
  dF1     *= half;                                 //  3   0   0      39   -   -
  dF1     += nbdy::c1(this);                       //  0   3   0       -  33   -
  dF2     *= third;                                //  3   0   0      42   -   -
  dF1     += dF2;                                  //  0   3   0       -  36   -
  S->pot()-= nbdy::c0(this);                       //  0   1   0       -  37   -
  S->pot()-= dX*dF1;                               //  3   1   -      45  38   -
}
////////////////////////////////////////////////////////////////////////////////
