// -*- C++ -*-                                                                 |
//-----------------------------------------------------------------------------+
//                                                                             |
// addgravity.cc                                                               |
//                                                                             |
// copyright Walter Dehnen, 2001                                               |
// e-mail:   dehnen@mpia.de                                                    |
// address:  Max-Planck Institut fr Astronomie, Knigstuhl 69117 Heidelberg   |
//           Germany                                                           |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// adds gravity (pot & acc) to a snapshot                                      |
//                                                                             |
//-----------------------------------------------------------------------------+
//                                                                             |
// history:                                                                    |
//                                                                             |
// v 1.0.0  19/10/2001  WD  created. initial version reads all snapshots.      |
// v 1.0.1  23/10/2001  WD  added option times= to select time range(s)        |
//                                                                             |
//-----------------------------------------------------------------------------+

#ifndef ALLOW_NEMO
#error You need "ALLOW_NEMO" to compile addgravity
#endif

#include <body.h>                                     // bodies etc..           
#include <falcON.h>                                   // fakcIB                 
#include <nbdy/nmio.h>                                // WDs C++ NEMO I/O       

extern "C" {
#  include <nemo.h>                                   // NEMO basics            
}

//------------------------------------------------------------------------------
string defv[] = {
  "in=???\n            input file                      ",
  "out=???\n           output file                     ",
  "times=all\n         time range                      ",
  "eps=0.05\n          softening length                ",
  "kernel=21\n         softening kernel                ",
  "theta=0.6\n         tolerance parameter at M=M_tot  ",
  "Ncrit=6\n           max # bodies in un-split cells  ",
  "VERSION=1.0.1\n     23/October/2001 WD              ",
  NULL};
string usage = "addgravity -- adds pot & acc to a snapshot using\n"
               "falcON = Force ALgorithm with Complexity O(N)\n";
//------------------------------------------------------------------------------
void nemo_main()
{
  nbdy::nemo_in  IN (getparam("in"));
  nbdy::nemo_out OUT;
  unsigned       NCRIT(getiparam("Ncrit"));
  nbdy::io       READ;
  const nbdy::io WRITE = nbdy::io::mxv | nbdy::io::p | nbdy::io::a;
  nbdy::real     TIME;
  nbdy::bodies   BODIES;
  nbdy::falcON   FALCON(&BODIES,
			nbdy::real(getdparam("eps")),
			nbdy::real(getdparam("theta")),
			nbdy::kern(getiparam("kernel")));
  IN.read_history();
  while(IN.is_present(nbdy::nemo_io::snap)) {
    READ = BODIES.read_nemo_snapshot(IN,&TIME,nbdy::io::mxv,getparam("times"));
    if(READ & nbdy::io::mxv) {
      if(!OUT.is_open()) {
	OUT.open(getparam("out"));
	OUT.write_history();
      }
      BODIES.flag_all_as_sink();
      FALCON.grow(NCRIT);
      FALCON.approximate_gravity();
      BODIES.write_nemo_snapshot(OUT,&TIME,WRITE);
    }
  }
}
